% sandwichEpochs(): This sandwiches nth epoch with (n-1)th and (n+1)th.
%                  Assued to be used for creating 'longer epoch' for later
%                  ERSP/ITC computation (to assure 3 sec epoch length!)
%                  This only replicates time series and does not replicate 
%                  event markers redanduntly so that subsequent epoch
%                  extraction does not mess up the event structure.
%
% Usage:
%    >> [ALLEEG, EEG, CURRENTSET] = sandwichEpochs(ALLEEG, EEG, CURRENTSET);

% Author: Makoto Miyakoshi, JSPS/SCCN, INC, UCSD.
% History:
% 03/12/2013 ver 1.0 by Makoto. Created for Gabriella Cruz. 

% Copyright (C) 2013, Makoto Miyakoshi JSPS/SCCN,INC,UCSD
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

function [ALLEEG, EEG, CURRENTSET] = sandwichEpochs(ALLEEG, EEG, CURRENTSET)

unitTime = 1000/EEG.srate/1000;
epochSec = EEG.pnts*unitTime;

% prepare suffix and prefix
prefixEEG = circshift(EEG.data,   [0 0 -1]);
suffixEEG = circshift(EEG.data,   [0 0  1]);
prefixIC  = circshift(EEG.icaact, [0 0 -1]);
suffixIC  = circshift(EEG.icaact, [0 0  1]);

% concatinate suffix and prefix
EEG.data   = cat(2, prefixEEG, EEG.data, suffixEEG);
EEG.icaact = cat(2, prefixIC, EEG.icaact, suffixIC);

% take care of EEG.event
for n = 1:length(EEG.event)
    extraPointsSoFar = EEG.pnts*(1+(EEG.event(1,n).epoch-1)*2);
    EEG.event(1,n).latency = EEG.event(1,n).latency + extraPointsSoFar;
end

% take care of EEG structure
EEG.pnts  = size(EEG.data,2);
EEG.xmin  = EEG.xmin-epochSec;
EEG.xmax  = EEG.xmax+epochSec;
EEG.times = EEG.xmin:unitTime:EEG.xmax;

% show message
disp('Epochs are successfully concatinated with preceeding and subsequent ones.')

% generate a new set file in the next slot
[ALLEEG EEG CURRENTSET] = eeg_store(ALLEEG, EEG, 0);

return